         LIST P=16C54
;************************************************************ 
;       2-Wire Byte Read Program (118 bytes)
;
;       This program demonstrates how to interface a  
;       Microchip PIC 16C54 to a 24LCXX Serial EE device
;       and perform a random read operation. This program
;       will read 8 consecutive addresses in the 'random
;       read' mode.  This entails setting the address pointer
;       before doing read command for every address.
;
;       Another, more efficient method of reading consecutive
;       addresses is the 'sequential read' mode.  This involves
;       sending the control byte and address for the first byte
;       to read, then continuing to provide clocks for the next
;       addresses.  The device will automatically increment the
;       address.  An example of the sequential read mode is
;       provided in the '2wseqr.asm' file.
;
;       Timing is based on using the PIC in 'XT' mode
;       using a 4Mhz crystal.  Clock speeds to the serial EE 
;       will be approximately 60 Khz for this setup.
;
;       As an option, the user may connect a LED to pin 18
;       on the PIC (use about a 1K resistor in series) as an
;       acknowledge fail indicator.  This LED will come on if
;       the serial EE fails to acknowledge correctly.  
;
;        PIC to Serial EE Connections:
;
;           PIC          Serial EE
;       ------------    ------------
;       Pin 12 (RB6) -->  SCLK
;       Pin 13 (RB7) -->  SDATA
;
;       PIN 18 (RA1) -->  Acknowledge fail LED  (Optional)
;
;
;************************************************************
;           Register Definitions
;************************************************************
status  equ     3h     ; status register
port_a  equ     5h     ; port 5 (port a) used for LED display
port_b  equ     6h     ; port 6 (port b) used for data and 
                       ; clock lines
eeprom  equ    0ah     ; bit buffer
bycnt   equ    0bh     ; byte counter for read mode
addr    equ    0ch     ; address counter
datai   equ    0dh     ; data input register
datao   equ    0eh     ; data output register
slave   equ    0fh     ; device address 1010xxx0)
txbuf   equ    10h     ; transmit buffer
count   equ    11h     ; bit counter
bcount  equ    12h     ; byte counter
loops   equ    15h     ; delay loop counter
loops2  equ    16h     ; delay loop counter

;************************************************************
;           Bit Definitions
;************************************************************
di      equ    7       ; eeprom input bit
do      equ    6       ; eeprom output bit
sdata   equ    7       ; serial EE data line (port_b,pin 13)
sclk    equ    6       ; serail EE clock line (port_b,pin 12)
ackf    equ    1       ; acknowledge fail LED (port_a)
;************************************************************
;
        org    01ffh   ; set reset vector
        goto   PWRUP
        org    000h    ; 
        goto   PWRUP  
;
;**************************************************************
;       Start Bit Subroutine
;       this routine generates a start bit
;       (Low going data line while clock is high)          
;**************************************************************
;
BSTART
         bsf     port_b,sdata   ; make sure data is high
         movlw   b'00111111'
         tris    port_b         ; set data and clock lines for output
         bcf     port_b,sclk    ; make sure clock is low
         nop
         bsf     port_b,sclk    ; set clock high
         nop
         nop
         nop
         nop
         nop
         bcf     port_b,sdata   ; data line goes low during
                                ; high clock for start bit
         nop
         nop
         nop
         nop
         nop                    ; timing adjustment
         bcf     port_b,sclk    ; start clock train
         nop
         nop
         retlw   0
         ;
;************************************************************
;       Stop Bit Subroutine
;       This routine generates a stop bit
;       (High going data line while clock is high)
;************************************************************
BSTOP
         bcf     port_b,sdata   ; make sure data line is low
         movlw   b'00111111'    ;
         tris    port_b         ; set data/clock lines as outputs
         bcf     port_b,sdata   ; make sure data line is low
         nop
         nop
         nop
         bsf     port_b,sclk    ; set clock high
         nop
         nop    
         nop
         bsf     port_b,sdata   ; data goes high while clock high
                                ; for stop bit
         nop
         nop
         bcf     port_b,sclk    ; set clock low again
         nop
         nop
         nop
         retlw   0
;
;*************************************************************
;       BITOUT routine takes the bit of data in 'do' and
;       transmits it to the serial EE device
;*************************************************************
BITOUT
         movlw   b'00111111'    ; set data,clock as outputs
         tris    port_b
         btfss   eeprom,do      ; check for state of data bit to xmit
         goto    bitlow         ; low? go set data line low 
         bsf     port_b,sdata   ; high? set data line high 
         goto    clkout         ; go toggle the clock

bitlow   bcf     port_b,sdata   ; output a low bit     
clkout   bsf     port_b,sclk    ; set clock line high
         nop
         nop
         nop
         nop
         bcf     port_b,sclk    ; return clock line low
         retlw   0
;                                                              
;**************************************************************
;       BITIN routine reads one bit of data from the 
;       serial EE device and stores it in the bit 'di'
;**************************************************************
BITIN
         bsf     eeprom,di      ; assume input bit is high
         movlw   b'10111111'    ; make sdata an input line
         tris    port_b
         bsf     port_b,sdata   ; set sdata line for input
         bsf     port_b,sclk    ; set clock line high
         nop                    ; just sit here a sec
         nop
         nop
         nop
         nop                    ;
         btfss   port_b,sdata   ; read the data bit
         bcf     eeprom,di      ; input bit was low, set 'di' accordingly
         bcf     port_b,sclk    ; set clock line low
                                ;
         retlw   0              ; 
;
;****************************************************************
;       Transmit Data Subroutine
;       This routine takes the byte of data stored in the
;       'datao' register and transmits it to the serial EE device.
;       It will then send 1 more clock to the serial EE for the
;       acknowledge bit.  If the ack bit from the part was low
;       then the transmission was sucessful.  If it is high, then
;       the device did not send a proper ack bit and the ack
;       fail LED will be turned on.
;****************************************************************
TX
         movlw   .8
         movwf   count          ; set the #bits to 8
         ;
TXLP
         bcf     eeprom,do      ; assume bit out is low
         btfsc   txbuf,7        ; is bit out really low?
         bsf     eeprom,do      ; no, set it high
         call    BITOUT         ; send the bit to serial EE
         rlf     txbuf          ; rotate txbuf left
         decfsz  count          ; 8 bits done?
         goto    TXLP           ; no - go again
         call    BITIN          ; read ack bit
         btfsc   eeprom,di      ; check ack bit
         bsf     port_a,ackf    ; set acknowledge fail LED if the
                                ; device did not pull data low
         ;
         retlw   0
;                    
;****************************************************************
;       Receive data routine
;       This routine reads one byte of data from the part
;       into the 'datai' register.  It then sends a high 
;       ack bit to indicate that no more data is to be read
;****************************************************************
RX
         clrf    datai          ; clear input buffer
         movlw   .8             ; set # bits to 8
         movwf   count      
         bcf     status,0       ; make sure carry bit is low
RXLP     rlf     datai          ; rotate datai 1 bit left
         call    BITIN          ; read a bit
         btfsc   eeprom,di
         bsf     datai,0        ; set bit 0 if necessary
         decfsz  count          ; 8 bits done?
         goto    RXLP           ; no, do another
         bsf     eeprom,do      ; set ack bit = 1
         call    BITOUT         ; to finish transmission
         retlw   0
;    
;****************************************************************
;       Power up routine
;       This is the program entry point.  I/O line status is
;       set for port A here.
;****************************************************************
PWRUP
          movlw  b'00000000'
          tris   port_a         ; set port A as all output
          clrf   port_a         ; all output lines low
          goto   READ
;
;**************************************************************
;       READ (read routine)
;       This routine reads 8 consecutive addresses of the
;       serial EE device starting at address 00 in the
;       random access mode (non-sequential read). Reading 
;       the device using the random access mode
;       requires that the address pointer be set for every 
;       byte before the read takes place. The address pointer
;       is set by sending a 'write mode' control byte with the
;       address of the location to read.  
;***************************************************************
READ
;
         bcf     port_a,ackf ; clear the ack fail LED if on
         movlw   .8          ; set number of bytes to read as 8
         movwf   bcount      ; 
         clrf    addr        ; set starting address to 00 
                             ;
rbyte    call    BSTART      ; generate start bit
                             ;
                             ; now send the write control byte and
                             ; address to set the pointer
                             ;
         movlw   b'10100000' ; get slave address (write mode)
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it        
         movf    addr,w      ; get word address
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it
                             ;
                             ; now read one byte from the part
                             ;
         call    BSTART      ; generate start bit
         movlw   b'10100001' ; get slave address and read mode
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and transmit it
         call    RX          ; read 1 byte from serial EE
         call    BSTOP       ; send stop bit to end transmission
         incf    addr        ; add 1 to address counter
         decfsz  bcount      ; yes, are all 8 bytes read?
         goto    rbyte       ; no, do another byte
         goto    READ        ; yes, start all over
;
END              
         


