         LIST P=16C54
;************************************************************ 
;       2-Wire Sequential Read Program (120 bytes)
;
;       This program demonstrates how to interface a  
;       Microchip PIC 16c54 to a 24LCXX Serial EE device
;       and perform a sequential read operation.  A sequential
;       read involves setting the address pointer once and then
;       using the auto-increment ability of the part to read
;       consecutive addresses by simply providing more clocks.
;
;       Timing is based on using the PIC in 'XT' mode
;       using a 4Mhz crystal.  Clock speeds to the serial EE 
;       will be approximately 60 Khz for this setup.
;
;       As an option, the user may connect a LED to pin 18
;       on the PIC (use about a 1K resistor in series) as an
;       acknowledge fail indicator.  This LED will come on if
;       the serial EE fails to acknowledge correctly.  
;
;        PIC to Serial EE Connections:
;
;           PIC          Serial EE
;       ------------    ------------
;       Pin 12 (RB6) -->  SCLK
;       Pin 13 (RB7) -->  SDATA
;
;       PIN 18 (RA1) -->  Acknowledge fail LED  (Optional)
;
;************************************************************
;           Register Definitions
;************************************************************
status  equ     3h     ; status register
port_a  equ     5h     ; port 5 (port_a) used for LEDs
port_b  equ     6h     ; port 6 (port b) used for data and 
                       ; clock lines
eeprom  equ    0ah     ; bit buffer
bycnt   equ    0bh     ; byte counter for read mode
addr    equ    0ch     ; address counter
datai   equ    0dh     ; data input register
datao   equ    0eh     ; data output register
slave   equ    0fh     ; device address (1010xxx0)
txbuf   equ    10h     ; transmit buffer
count   equ    11h     ; bit counter
bcount  equ    12h     ; byte counter
loops   equ    15h     ; delay loop counter
loops2  equ    16h     ; delay loop counter
;************************************************************
;           Bit Definitions
;************************************************************
di      equ    7       ; eeprom input bit
do      equ    6       ; eeprom output bit
sdata   equ    7       ; serial EE data line (port_b,pin 13)
sclk    equ    6       ; serail EE clock line (port_b,pin 12)
ackf    equ    1       ; acknowledge fail LED, port_a
;************************************************************
;
;
        org    01ffh   ; set reset vector
        goto   PWRUP
        org    000h    ; 
        goto   PWRUP  
;
;**************************************************************
;       Start Bit Subroutine
;       this routine generates a start bit
;       (Low going data line while clock is high)          
;**************************************************************
;
BSTART
         bsf     port_b,sdata   ; make sure data is high
         movlw   b'00111111'
         tris    port_b         ; set data and clock lines for output
         bcf     port_b,sclk    ; make sure clock is low
         nop
         bsf     port_b,sclk    ; set clock high
         nop
         nop
         nop
         nop
         nop
         bcf     port_b,sdata   ; data line goes low during
                                ; high clock for start bit
         nop
         nop
         nop
         nop
         nop                    ; timing adjustment
         bcf     port_b,sclk    ; start clock train
         nop
         nop
         retlw   0
         ;
         ;    End of Subroutine
;************************************************************
;       Stop Bit Subroutine
;       This routine generates a stop bit
;       (High going data line while clock is high)
;************************************************************
BSTOP
         movlw   b'00111111'    ;
         tris    port_b         ; set data/clock lines as outputs
         bcf     port_b,sdata   ; make sure data line is low
         nop
         nop
         nop
         bsf     port_b,sclk    ; set clock high
         nop
         nop    
         nop
         bsf     port_b,sdata   ; data goes high while clock high
                                ; for stop bit
         nop
         nop
         bcf     port_b,sclk    ; set clock low again
         nop
         nop
         nop
         retlw   0
;
;           End of Subroutine
;*************************************************************
;       BITOUT routine takes one bit of data in 'do' and
;       transmits it to the serial EE device
;*************************************************************
BITOUT
         movlw   b'00111111'    ; set data,clock as outputs
         tris    port_b
         btfss   eeprom,do      ; check for state of data bit to xmit
         goto    bitlow         ; 
         bsf     port_b,sdata   ; set data line high 
         goto    clkout         ; go toggle the clock

bitlow   bcf     port_b,sdata   ; output a low bit     
clkout   bsf     port_b,sclk    ; set clock line high
         nop
         nop
         nop
         nop
         bcf     port_b,sclk    ; return clock line low
         retlw   0
;
;                   End of Subroutine
;                                                              
;**************************************************************
;       BITIN routine reads one bit of data from the 
;       serial EE device and stores it in 'di'
;**************************************************************
BITIN
         bsf     eeprom,di      ; assume input bit is high
         movlw   b'10111111'    ; make sdata an input line
         tris    port_b
         bsf     port_b,sdata   ; set sdata line for input
         bsf     port_b,sclk    ; set clock line high
         nop                    ; just sit here a sec
         nop
         nop
         nop
         nop                    ;
         btfss   port_b,sdata   ; read the data bit
         bcf     eeprom,di      ; input bit was low, set 'di' accordingly
         bcf     port_b,sclk    ; set clock line low
                                ;
         retlw   0
;
;****************************************************************
;       Transmit Data Subroutine
;       This routine takes the byte of data stored in the
;       'datao' register and transmits it to the serial EE device.
;       It will then send 1 more clock to the serial EE for the
;       acknowledge bit.  If the ack bit from the part was low
;       then the transmission was sucessful.  If it is high, then
;       the device did not send a proper ack bit and the ack
;       fail LED will be turned on.
;****************************************************************
TX
         movlw   .8
         movwf   count          ; set the #bits to 8
         ;
TXLP
         bcf     eeprom,do      ; assume bit out is low
         btfsc   txbuf,7        ; is bit out really low?
         bsf     eeprom,do      ; no, set it high
         call    BITOUT         ; send the bit to serial EE
         rlf     txbuf          ; rotate txbuf left
         decfsz  count          ; 8 bits done?
         goto    TXLP           ; no - go again
         call    BITIN          ; read ack bit
         btfsc   eeprom,di      ; check ack bit
         bsf     port_a,ackf    ; set acknowledge fail LED if the
                                ; device did not pull data low
         ;
         retlw   0
;                    
;****************************************************************
;       Receive data Routine
;       This routine reads one byte of data from the part
;       into the 'datai' register.  It then sends a high 
;       ack bit to indicate that no more data is to be read
;****************************************************************
RX
         movlw   .8             ; set # bits to 8
         movwf   count      
         clrf    datai          ; clear input register
         bcf     status,0       ; make sure carry bit is low
RXLP     rlf     datai          ; rotate datai 1 bit left
         call    BITIN          ; read a bit
         btfsc   eeprom,di
         bsf     datai,0        ; set bit 0 if necessary
         decfsz  count          ; 8 bits done?
         goto    RXLP           ; no, do another
         retlw   0
;    
;****************************************************************
;       Power up routine
;       This is the program entry point, which in this case simply
;       sets the port_a I/O lines and directs control to the
;       Read routine.
;****************************************************************
PWRUP
          movlw  b'00000000'
          tris   port_a         ; set port A as all output
          clrf   port_a         ; all output lines low
          goto   READ

;
;**************************************************************
;       READ (sequential read routine)
;
;       This routine reads 8 consecutive addresses of the
;       serial EE device starting at address 00 in the
;       sequential read mode. Reading in this mode is more
;       efficient than the random read mode as the control byte
;       and address have to be sent only once at the beginning
;       of the sequence.  As many consecutive addresses as
;       needed can then be read from the part until a stop bit is
;       sent.  In the read mode, the PIC must send the acknowledge
;       bit after every 8 data bits from the device.  When the
;       last byte needed has been read, then the controller will
;       send a high acknowledge bit and then a stop bit to halt
;       transmission from the device. 
;***************************************************************
READ
                             ;
         bcf     port_a,ackf ; clear the ack fail LED if on
         movlw   .8
         movwf   bcount      ; set number of bytes to read as 8
         movlw   b'10100000' ; set slave address and write mode
         movwf   slave
         clrf    addr        ; set starting address to 00 
                             ;
         call    BSTART      ; generate start bit
         movf    slave,w     ; get slave address
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it        
         movf    addr,w      ; get word address
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it        
         call    BSTART      ; generate start bit
         movlw   b'10100001' ; get slave address and read mode
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and transmit it
                             ;
rbyte    call    RX          ; read 1 byte from device
         decfsz  bcount      ; are all 8 bytes read?
         goto    lowack      ; no, send low ack and do another
         bsf     eeprom,do   ; yes, send high ack bit 
         call    BITOUT      ; to stop tranxmission
         call    BSTOP       ; and send a stop bit 
         goto    READ        ; start all over
  
lowack   bcf     eeprom,do   ; send low ack bit 
         call    BITOUT      ; to continue transmission
         goto    rbyte       ; and read another byte
;
END              
         


